<?php

namespace Supablox;

const CANVAS_WRAPPER_ID = 'supablox-canvas';
const CANVAS_OVERLAY_ID = 'supablox-canvas-overlays';

if ( is_canvas() ) {
	global $post;

	require_once ABSPATH . WPINC . '/pluggable.php';

	if ( ! wp_verify_nonce( $_GET['nonce'], 'wp_rest' ) ) {
		wp_die( 'Invalid nonce.', 'Invalid nonce', [ 'response' => 403 ] );
	} elseif ( $post && ! current_user_can( 'edit_post', $post ) ) {
		wp_die( 'Unauthorized.', 'Unauthorized', [ 'response' => 403 ] );
	} elseif ( ! current_user_can( 'edit_posts' ) ) {
		wp_die( 'Unauthorized.', 'Unauthorized', [ 'response' => 403 ] );
	}

	// phpcs:disable WordPress.Security.NonceVerification.Recommended

	remove_action( 'wp_enqueue_scripts', 'wp_enqueue_global_styles' );
	remove_action( 'wp_footer', 'wp_enqueue_global_styles', 1 );

	add_action( 'show_admin_bar', '__return_false' );
	add_action( 'init', __NAMESPACE__ . '\canvas_init' );
	add_action( 'wp_enqueue_scripts', __NAMESPACE__ . '\enqueue_canvas_assets' );
	add_filter( 'language_attributes', __NAMESPACE__ . '\canvas_html_classes' );
	add_filter( 'body_class', __NAMESPACE__ . '\canvas_body_classes' );
	add_filter( 'the_title', __NAMESPACE__ . '\canvas_post_title', 10, 2 );
	add_filter( 'the_posts', __NAMESPACE__ . '\replace_canvas_main_post_content', 10, 2 );
	add_filter( 'post_content', __NAMESPACE__ . '\replace_canvas_post_content', 10, 3 );
	add_filter( 'get_post_metadata', __NAMESPACE__ . '\override_post_metadata', 10, 4 );

	// Third-party compatibility.
	add_filter( 'woocommerce_checkout_redirect_empty_cart', '__return_false', 999 );
}

function canvas_init() {
	do_action( 'supablox_canvas_init' );
}

function enqueue_canvas_assets() {
	wp_enqueue_style( 'supablox/block-editor' );
	wp_enqueue_style( 'supablox/components' );
	wp_enqueue_style( 'supablox/editor' );

	foreach ( \WP_Block_Type_Registry::get_instance()->get_all_registered() as $block_type ) {
		foreach ( $block_type->script_handles as $handle ) {
			wp_enqueue_script( $handle );
		}
		foreach ( $block_type->style_handles as $handle ) {
			wp_enqueue_style( $handle );
		}
	}

	do_action( 'supablox_enqueue_canvas_assets' );
}

function canvas_html_classes( string $output ) {
	$class_regex = '/class=["\']([^"\']*)["\']/';

	if ( preg_match( $class_regex, $output, $matches ) ) {
		$classes = explode( ' ', $matches[1] );
		$classes = array_unique( [ ...$classes, 'block-editor-iframe__html' ] );
		$output = preg_replace( $class_regex, 'class="' . implode( ' ', $classes ) . '"', $output );
	} else {
		$output .= ' class="block-editor-iframe__html"';
	}

	return $output;
}

function canvas_body_classes( array $classes ) {
	$classes[] = 'block-editor-iframe__body';
	$classes[] = 'editor-styles-wrapper';
	$classes[] = 'sb-editor-canvas';
	$classes[] = 'is-classic-theme';
	$classes[] = 'is-root-container';
	$classes[] = 'block-editor-block-list__layout';
	$classes[] = 'admin-color-modern';
	return $classes;
}

function canvas_post_title( string $post_title, int $post_id ) {
	if ( is_current_canvas_post( $post_id ) ) {
		return '<span class="sb-editor-canvas-title">' . $post_title . '</span>';
	}
	return $post_title;
}

function replace_canvas_main_post_content( array $posts, \WP_Query $query ) {
	if ( ! $query->is_main_query() ) {
		return $posts;
	}

	// Get current post if no posts are found. Happens when editing a draft.
	if ( empty( $posts ) ) {
		if ( $post = get_current_canvas_post() ) {
			$posts[] = $post;
		}
	}

	// Check if this post is being edited in builder.
	if ( count( $posts ) === 1 && is_current_canvas_post( $posts[0]->ID ) ) {
		$posts[0]->post_content = '<div id="' . CANVAS_WRAPPER_ID . '" class="is-classic-theme-wrapper"></div>';
		$posts[0]->post_excerpt = '<div class="sb-editor-canvas-excerpt">' . $posts[0]->post_excerpt . '</div>';
	}

	return $posts;
}

function replace_canvas_post_content( string $content, int $post_id, string $context ) {
	return is_current_canvas_post( $post_id )
		? '<div id="' . CANVAS_WRAPPER_ID . '" class="is-classic-theme-wrapper"></div>'
		: $content;
}

function override_post_metadata( mixed $value, int $object_id, string $meta_key, bool $single ) {
	if ( array_key_exists( $meta_key, $_GET ) ) {
		return wp_unslash( $_GET[ $meta_key ] );
	}
	return $value;
}

function get_current_canvas_post() {
	$post = get_post( get_current_canvas_post_id() );

	if ( $post instanceof \WP_Post ) {
		return $post;
	}

	return null;
}

function is_current_canvas_post( int $post_id ) {
	return get_current_canvas_post_id() === $post_id;
}

function get_current_canvas_post_id() {
	return isset( $_GET['post_id'] ) ? intval( $_GET['post_id'] ) : -1;
}
