<?php

namespace Supablox;

const EDITOR_SCRIPT_OVERRIDES = [
	'wp-private-apis' => 'wp-private-apis.js',
];

const EDITOR_SCRIPT_PROXIES = [
	'wp-block-editor' => 'wp-block-editor.js',
	'wp-components' => 'wp-components.js',
	'wp-plugins' => 'wp-plugins.js',
];

const PROXY_SCRIPT_PREFIX = 'supablox-';

if ( is_editor() ) {
	add_action( 'admin_init', __NAMESPACE__ . '\editor_init' );
	add_action( 'admin_head', __NAMESPACE__ . '\editor_admin_head' );
	add_action( 'current_screen', __NAMESPACE__ . '\editor_screen' );
	add_action( 'wp_default_scripts', __NAMESPACE__ . '\override_editor_scripts', 11 );
	add_action( 'admin_enqueue_scripts', __NAMESPACE__ . '\enqueue_editor_assets' );
	add_filter( 'use_block_editor_for_post_type', __NAMESPACE__ . '\use_block_editor_for_post_type', 11, 2 );
	add_filter( 'gutenberg_can_edit_post_type', __NAMESPACE__ . '\use_block_editor_for_post_type', 11, 2 );
	add_filter( 'block_editor_rest_api_preload_paths', __NAMESPACE__ . '\editor_preload_rest_api_paths', 10, 2 );
	add_filter( 'admin_title', __NAMESPACE__ . '\editor_admin_title', 10, 2 );
	add_filter( 'admin_body_class', __NAMESPACE__ . '\editor_body_class' );
	add_filter( 'get_user_option_admin_color', __NAMESPACE__ . '\get_user_admin_color', 11 );
	add_action( 'admin_enqueue_scripts', __NAMESPACE__ . '\enqueue_editor_admin_assets' );
	add_action( 'enqueue_block_assets', __NAMESPACE__ . '\enqueue_editor_block_asset' );
	add_action( 'wp_print_scripts', __NAMESPACE__ . '\process_editor_scripts', 100 );
	add_action( 'init', __NAMESPACE__ . '\add_block_editor_script_data', 100 );
	add_filter( 'screen_options_show_screen', '__return_false' );
}

function editor_init() {
	global $post_type, $post_type_object, $post, $wp_version;

	$plugin_info = get_plugin_data( \Supablox\FILE );
	$requires_wp = $plugin_info['RequiresWP'] ?? '6.6';

	if ( version_compare( $wp_version, $requires_wp, '<' ) ) {
		wp_die(
			sprintf(
				wp_kses(
					// translators: %1$s: WordPress version, %2$s: Update URL.
					__( 'Supablox requires WordPress %1$s or higher. <a href="%2$s">Please upgrade your WordPress installation.</a>', 'supablox' ),
					[
						'a' => [ 'href' => [] ],
					]
				),
				esc_attr( $requires_wp ),
				esc_url( admin_url( 'update-core.php' ) )
			)
		);
	}

	// phpcs:disable WordPress.WP.GlobalVariablesOverride,WordPress.Security

	$post_id = isset( $_GET['postId'] ) ? sanitize_key( $_GET['postId'] ) : 0;
	$post_type = isset( $_GET['postType'] ) ? sanitize_key( $_GET['postType'] ) : 'post';
	$post_type_object = get_post_type_object( $post_type );
	$post = get_post( $post_id );

	// phpcs:enable WordPress.WP.GlobalVariablesOverride,WordPress.Security

	if ( is_numeric( $post_id ) && ! $post ) {
		wp_die( esc_html__( 'You attempted to edit an item that does not exist. Perhaps it was deleted?', 'default' ) );
	}

	$custom_settings = [
		'siteUrl' => site_url(),
		'postsPerPage' => get_option( 'posts_per_page' ),
		'styles' => get_block_editor_theme_styles(),
		'defaultTemplateTypes' => array_map(
			function ( $template_type, $slug ) {
				$template_type['slug'] = (string) $slug;
				return $template_type;
			},
			get_default_block_template_types(),
			array_keys( get_default_block_template_types() )
		),
		'defaultTemplatePartAreas'  => get_allowed_block_template_part_areas(),
		'supportsLayout' => wp_theme_has_theme_json(),
		'supportsTemplateMode' => current_theme_supports( 'block-templates' ),
		'supportsTemplatePartsMode' => ! wp_is_block_theme() && current_theme_supports( 'block-template-parts' ),
		'supablox' => [
			'id' => 'supablox',
			'defaultRoute' => get_default_editor_route(),
			'isBlockTheme' => wp_is_block_theme(),
			'themeName' => get_template(),
			'settingsUrl' => admin_url( 'options-general.php?page=supablox' ),
		],
	];

	$custom_settings['__experimentalAdditionalBlockPatterns'] = \WP_Block_Patterns_Registry::get_instance()->get_all_registered( true );
	$custom_settings['__experimentalAdditionalBlockPatternCategories'] = \WP_Block_Pattern_Categories_Registry::get_instance()->get_all_registered( true );

	$block_editor_context = new \WP_Block_Editor_Context( [ 'name' => 'supablox' ] );
	$editor_settings = get_block_editor_settings( $custom_settings, $block_editor_context );
	$editor_settings = array_merge( $editor_settings, get_editor_settings_for_post( $post_type, $post_id ) );

	$active_global_styles_id = \WP_Theme_JSON_Resolver::get_user_global_styles_post_id();
	$active_theme = get_stylesheet();

	$navigation_rest_route = rest_get_route_for_post_type_items(
		'wp_navigation'
	);

	$preload_paths = [
		'/wp/v2/types?context=view',
		'/wp/v2/types/wp_template?context=edit',
		'/wp/v2/types/wp_template-part?context=edit',
		'/wp/v2/templates?context=edit&per_page=-1',
		'/wp/v2/template-parts?context=edit&per_page=-1',
		'/wp/v2/themes?context=edit&status=active',
		'/wp/v2/global-styles/' . $active_global_styles_id,
		'/wp/v2/global-styles/' . $active_global_styles_id . '?context=edit',
		'/wp/v2/global-styles/themes/' . $active_theme,
		[ $navigation_rest_route, 'OPTIONS' ],
		[ '/wp/v2/users/me', 'GET' ],
		[ '/wp/v2/settings', 'GET' ],
		[ '/wp/v2/settings', 'OPTIONS' ],
		[ '/wp/v2/media', 'OPTIONS' ],
		[ rest_get_route_for_post_type_items( 'attachment' ), 'OPTIONS' ],
		[ rest_get_route_for_post_type_items( 'page' ), 'OPTIONS' ],
		[ rest_get_route_for_post_type_items( 'wp_block' ), 'OPTIONS' ],
		[ rest_get_route_for_post_type_items( 'wp_template' ), 'OPTIONS' ],
		[
			add_query_arg(
				[
					'context' => 'edit',
					'per_page' => 100,
					'order' => 'desc',
					'orderby' => 'date',
					'status[0]' => 'publish',
					'status[1]' => 'draft',
				],
				$navigation_rest_route
			),
			'GET',
		],
	];

	block_editor_rest_api_preload( $preload_paths, $block_editor_context );

	wp_add_inline_script(
		'wp-blocks',
		sprintf(
			'for ( const source of %s ) { wp.blocks.registerBlockBindingsSource?.( source ); }', // Remove optional chaining when WP 6.7 is the minimum version.
			wp_json_encode(
				array_map(
					function ( $source ) {
						return [
							'name' => $source->name,
							'label' => $source->label,
							'usesContext' => $source->uses_context,
						];
					},
					array_values( get_all_registered_block_bindings_sources() )
				)
			)
		)
	);

	wp_add_inline_script(
		'wp-blocks',
		sprintf( 'wp.blocks.setCategories( %s );', wp_json_encode( isset( $editor_settings['blockCategories'] ) ? $editor_settings['blockCategories'] : [] ) ),
		'after'
	);

	wp_add_inline_script(
		'wp-blocks',
		sprintf( 'wp.blocks.unstable__bootstrapServerSideBlockDefinitions( %s );', wp_json_encode( get_block_editor_server_block_settings() ) )
	);

	wp_add_inline_script(
		'wp-preferences',
		sprintf(
			'wp.data.dispatch( wp.preferences.store ).setDefaults( "%s", %s );',
			esc_js( PREFERENCES_NAME ),
			wp_json_encode( DEFAULT_PREFERENCES )
		),
		'after'
	);

	wp_add_inline_script(
		'supablox/editor',
		sprintf(
			'window.supabloxEditorData = %s;',
			wp_json_encode(
				[
					'wordpressVersion' => $wp_version,
				]
			)
		),
		'before'
	);

	wp_add_inline_script(
		'supablox/editor',
		sprintf( 'wp.domReady( () => supablox.editor.initialize( %s ) );', wp_json_encode( $editor_settings ) )
	);

	wp_enqueue_media( [ 'post' => is_numeric( $post_id ) ? $post_id : null ] );
	wp_tinymce_inline_scripts();
	wp_enqueue_editor();

	wp_enqueue_script( 'heartbeat' );
	wp_enqueue_script( 'wp-dom-ready' );
	wp_enqueue_script( 'wp-format-library' );
	wp_enqueue_script( 'supablox-block-editor' );
	wp_enqueue_script( 'supablox/editor' );
	wp_enqueue_style( 'wp-components' );
	wp_enqueue_style( 'wp-edit-site' );
	wp_enqueue_style( 'wp-format-library' );
	wp_enqueue_style( 'supablox/editor' );
	wp_enqueue_style( 'supablox-block-editor' );

	remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
	remove_action( 'admin_print_styles', 'print_emoji_styles' );

	do_action( 'supablox_editor_init' );
}

function editor_admin_head() {
	echo '<meta name="theme-color" content="#1e1e1e" />' . PHP_EOL;
}

function editor_screen( \WP_Screen $current_screen ) {
	$current_screen->is_block_editor( true );
}

function editor_preload_rest_api_paths( array $preload_paths, \WP_Block_Editor_Context $block_editor_context ) {
	global $post_type, $post;

	if ( empty( $post ) || $block_editor_context->name !== 'supablox' ) {
		return $preload_paths;
	}

	$rest_path = rest_get_route_for_post( $post->ID );
	$settings_url = add_query_arg(
		[
			'post_type' => $post_type,
			'post_id' => $post->ID,
		],
		'/supablox/v1/settings'
	);

	$preload_paths = array_merge(
		$preload_paths,
		[
			[ $rest_path , 'OPTIONS' ],
			[ add_query_arg( 'context', 'edit', $rest_path ), 'GET' ],
			[ sprintf( '%s/autosaves?context=edit', $rest_path ), 'GET' ],
			[ sprintf( '/wp/v2/types/%s?context=edit', $post_type ), 'GET' ],
			[ $settings_url, 'GET' ],
			[ $settings_url, 'OPTIONS' ],
		]
	);

	if ( $post_type !== 'wp_template' && ! empty( $post->post_name ) ) {
		$template_slug = $post_type . '-' . $post->post_name;
		$preload_paths[] = [ add_query_arg( 'slug', $template_slug, '/wp/v2/templates/lookup' ), 'GET' ];
	}

	return $preload_paths;
}

function get_editor_settings_for_post( $post_type, $post_id ) {
	$post = get_post( $post_id );
	$settings = [];

	if ( $post ) {
		$post_type_object = get_post_type_object( $post->post_type );
		$is_new_post = $post->post_status === 'auto-draft';
	} else {
		$post_type_object = get_post_type_object( $post_type );
		$post = get_default_post_to_edit( $post_type, false );
		$is_new_post = true;
	}

	if ( ! $post_type_object ) {
		return rest_ensure_response( $settings );
	}

	add_filter( 'pre_get_block_templates', '__return_empty_array' );
	$available_templates = wp_get_theme()->get_page_templates( null, $post->post_type );
	remove_filter( 'pre_get_block_templates', '__return_empty_array' );

	$default_page_template_title = apply_filters( 'default_page_template_title', __( 'Default template', 'default' ), 'rest-api' );
	$available_templates = ! empty( $available_templates )
		? array_replace(
			[ '' => $default_page_template_title ],
			$available_templates
		)
		: [ '' => $default_page_template_title ];

	$settings['availableTemplates'] = $available_templates;
	$settings['titlePlaceholder'] = apply_filters( 'enter_title_here', __( 'Add title', 'default' ), $post );
	$settings['bodyPlaceholder'] = apply_filters( 'write_your_story', __( 'Type / to choose a block', 'default' ), $post );
	$settings['postLock'] = get_post_lock_details( $post );
	$settings['postLockUtils'] = [
		'nonce' => wp_create_nonce( 'lock-post_' . $post->ID ),
		'unlockNonce' => wp_create_nonce( 'update-post_' . $post->ID ),
		'ajaxUrl' => admin_url( 'admin-ajax.php' ),
	];

	if ( ! empty( $post_type_object->template ) ) {
		$settings['template'] = $post_type_object->template;
		$settings['templateLock'] = ! empty( $post_type_object->template_lock ) ? $post_type_object->template_lock : false;
	}

	// If there's no template set on a new post, use the post format, instead.
	if ( $is_new_post && ! isset( $settings['template'] ) && $post->post_type === 'post' ) {
		$post_format = get_post_format( $post );
		if ( in_array( $post_format, [ 'audio', 'gallery', 'image', 'quote', 'video' ], true ) ) {
			$settings['template'] = [ [ "core/$post_format" ] ];
		}
	}

	return $settings;
}

function override_editor_scripts( \WP_Scripts $scripts ) {
	foreach ( EDITOR_SCRIPT_PROXIES as $handle => $filename ) {
		$script = $scripts->query( $handle, 'registered' );

		if ( ! $script || str_starts_with( $script->src, ASSETS_URL ) ) return;

		$new_handle = PROXY_SCRIPT_PREFIX . $handle;

		$scripts->add( $new_handle, $script->src, $script->deps, $script->ver, $script->args );

		foreach ( $script->extra as $key => $value ) {
			$scripts->add_data( $new_handle, $key, $value );
		}

		$asset_info = get_asset_info( $filename );

		// Remove the overridden script from the dependencies to avoid a circular dependency.
		if ( ( $key = array_search( $handle, $asset_info['dependencies'], true ) ) !== false ) {
			unset( $asset_info['dependencies'][ $key ] );
		}

		$asset_info['dependencies'][] = $new_handle;

		if ( is_development() ) {
			$asset_info['dependencies'][] = 'supablox/runtime';
		}

		$script->src = ASSETS_URL . $filename;
		$script->ver = $asset_info['version'];
		$script->deps = $asset_info['dependencies'];
		$script->extra = [];
	}

	foreach ( EDITOR_SCRIPT_OVERRIDES as $handle => $filename ) {
		$script = $scripts->query( $handle, 'registered' );

		if ( ! $script || str_starts_with( $script->src, ASSETS_URL ) ) return;

		$asset_info = get_asset_info( $filename );
		$script->src = ASSETS_URL . $filename;
		$script->ver = $asset_info['version'];

		if ( is_development() ) {
			$script->deps[] = 'supablox/runtime';
		}
	}
}

/**
 * Process the editor scripts to prevent circular dependencies.
 */
function process_editor_scripts() {
	$wp_include_dir = '/' . WPINC . '/';
	$gutenberg_build_url = function_exists( 'gutenberg_url' ) ? gutenberg_url( 'build/' ) : '';

	foreach ( EDITOR_SCRIPT_PROXIES as $filename ) {
		$asset_info = get_asset_info( $filename );

		foreach ( $asset_info['dependencies'] as $handle ) {
			$script = wp_scripts()->query( $handle, 'registered' );

			if (
				! $script ||
				! str_starts_with( $script->src, $wp_include_dir ) ||
				! str_starts_with( $script->src, $gutenberg_build_url )
			) {
				continue;
			}

			foreach ( $script->deps as &$dep ) {
				if ( array_key_exists( $dep, EDITOR_SCRIPT_PROXIES ) ) {
					$dep = PROXY_SCRIPT_PREFIX . $dep;
				}
			}
		}
	}
}

function enqueue_editor_assets() {
	do_action( 'enqueue_block_editor_assets' );
}

/**
 * Use the block editor for all post types that support Supablox.
 */
function use_block_editor_for_post_type( bool $use_block_editor, string $post_type ) {
	return post_type_supports( $post_type, 'supablox' ) ? true : $use_block_editor;
}

function editor_admin_title( $admin_title, $title ) {
	// phpcs:disable WordPress.Security.NonceVerification.Recommended
	if ( isset( $_GET['postId'] ) && ( $post = get_post( sanitize_key( $_GET['postId'] ) ) ) ) {
		return sprintf(
			// translators: %1$s: Post title, %2$s: Site title.
			esc_html__( '%1$s ‹ %2$s ‹ Supablox — WordPress', 'supablox' ),
			$post->post_title,
			get_bloginfo( 'name' )
		);
	}
	return $admin_title;
}

function editor_body_class( string $classes ) {
	return "$classes page-supablox is-fullscreen-mode";
}

function get_user_admin_color() {
	return 'modern';
}

function get_default_template_types() {
	return array_map(
		function ( $template_type, $slug ) {
			$template_type['slug'] = (string) $slug;
			return $template_type;
		},
		get_default_block_template_types(),
		array_keys( get_default_block_template_types() )
	);
}

function get_default_editor_route() {
	if ( $post_id = get_option( 'page_on_front' ) ) {
		return [
			'postType' => 'page',
			'postId' => $post_id,
		];
	} elseif ( $post_id = get_option( 'page_for_posts' ) ) {
		return [
			'postType' => 'page',
			'postId' => $post_id,
		];
	} elseif ( wp_is_block_theme() ) {
		$theme_slug = wp_get_theme()->get_stylesheet();

		if ( $home_template = get_block_template( "$theme_slug//home" ) ) {
			return [
				'postType' => 'wp_template',
				'postId' => $home_template->id,
			];
		}

		return [
			'postType' => 'wp_template',
			'postId' => "$theme_slug//index",
		];
	}
	return [];
}

function enqueue_editor_admin_assets() {
	wp_enqueue_style( 'supablox/editor' );
	wp_enqueue_style( 'supablox/wp-block-editor' );
	wp_enqueue_style( 'supablox/wp-components' );

	do_action( 'supablox_enqueue_assets' );
}

/**
 * Enqueues editor styles that should be loaded in the canvas iframe.
 */
function enqueue_editor_block_asset() {
	wp_enqueue_style( 'supablox/editor' );
	wp_enqueue_style( 'supablox/wp-block-editor' );
	wp_enqueue_style( 'supablox/wp-components' );
}

function add_block_editor_script_data() {
	$data = [
		'largeScreenWidth' => '1440px',
		'devices' => [
			'Mobile' => [
				'width' => 375,
				'height' => 660,
			],
			'Tablet' => [
				'width' => 850,
				'height' => 1024,
			],
			'Desktop' => [
				'width' => null,
				'height' => null,
			],
			'Large'   => [
				'width' => null,
				'height' => null,
			],
		],
	];

	$inline_script = sprintf( 'window.supabloxWpBlockEditorData = %s;', wp_json_encode( $data ) );

	wp_add_inline_script( 'wp-block-editor', $inline_script, 'before' );
}
