<?php

namespace Supablox\Packages\Admin;

final class Lemon_Squeezy_License extends License {

	public function init(): void {
		add_filter( 'pre_update_option_' . PACKAGE_NAME, [ $this, 'handle_activation' ], 10, 2 );
	}

	public function add_settings(): void {
		add_settings_field( 'license_key', 'Activation', [ $this, 'license_key_setting_callback' ], Settings_Page::ID, PACKAGE_NAME );
	}

	public function license_key_setting_callback() {
		$activation = $this->get_activation();
		$options = get_option( PACKAGE_NAME );
		$license_key = isset( $options['license_key'] ) ? $options['license_key'] : '';

		if ( $this->is_activated() ) {
			$date_format = get_option( 'date_format' );
			$time_format = get_option( 'time_format' );
			$created_at = strtotime( $activation['created_at'] );
			$instance_name = $activation['instance_name'];
			$key = $this->get_license_key();

			printf( '<input type="hidden" name="%s[license_key]" value="hidden">', esc_attr( PACKAGE_NAME ) );
			printf( '<span class="dashicons dashicons-admin-network" style="display:inline-block;height:30px;line-height:30px;font-size:16px;margin-right:-1px;padding-block:0;padding-inline:8px;background:rgb(186 221 192);"></span>' );
			printf( '<code style="display:inline-block;line-height:30px;padding-block:0;padding-inline:12px;margin-right:12px;">%s</code>', 'XXXX-' . esc_attr( substr( $key, 24, 12 ) ) );
			submit_button( 'Deactivate', '', sprintf( '%s[deactivate]', PACKAGE_NAME ), false );
			printf( '<p class="description" style="margin-top:8px;"><strong>%s</strong>, activated on %s</p>', esc_attr( $instance_name ), esc_attr( wp_date( $date_format . ' ' . $time_format, $created_at ) ) );
		} else {
			$errors = get_settings_errors( PACKAGE_NAME );

			printf( '<input class="regular-text" type="text" name="%s[license_key]" value="%s">', esc_attr( PACKAGE_NAME ), esc_attr( $license_key ) );

			if ( count( $errors ) ) {
				printf( '<p class="description" style="margin-top:8px;color:red;">%s</p>', wp_kses_post( $errors[0]['message'] ) );
			} elseif ( $license_key ) {
				printf( '<p class="description" style="margin-top:8px;color:red;">%s</p>', 'Invalid License Key' );
			} else {
				printf( '<p class="description" style="margin-top:8px;">%s</p>', 'Enter License Key' );
			}
		}
	}

	public function settings_sanitize_callback( array|null $input ): array {
		$license_key = isset( $input['license_key'] ) ? trim( $input['license_key'] ) : '';

		if ( isset( $input['deactivate'] ) || empty( $license_key ) ) {
			return [];
		}

		if ( $license_key === 'hidden' ) {
			return [
				'license_key' => $this->get_license_key(),
			];
		}

		return [
			'license_key' => $license_key,
		];
	}

	public function handle_activation( $new_value, $old_value ) {
		$new_license_key = trim( $new_value['license_key'] ?? '' );
		$license_key = $this->get_license_key();
		$instance_id = $this->get_instance_id();

		if ( $new_license_key && $new_license_key !== $license_key ) {
			$result = $this->activate( [ 'license_key' => $new_license_key ] );
			if ( ! is_wp_error( $result ) ) {
				$this->update_activation( $result );
			}
		} elseif ( $instance_id && $license_key && ! $new_license_key ) {
			$result = $this->deactivate();
			if ( ! is_wp_error( $result ) ) {
				$this->clear_activation();
			}
		} else {
			$result = null;
		}

		if ( is_wp_error( $result ) ) {
			add_settings_error( PACKAGE_NAME, 'license_key', $result->get_error_message(), 'error' );
		}

		return $new_value;
	}

	public function validate( array $args = null ): array|\WP_Error {
		$license_key = $this->get_license_key();
		$instance_id = $this->get_instance_id();

		if ( empty( $license_key ) ) {
			return new \WP_Error( 400, 'Invalid license key.' );
		}

		$data = Supablox_Api::get_instance()->get_license(
			[
				'license_key' => $license_key,
				'instance_id' => $instance_id,
			]
		);

		if ( isset( $data['valid'] ) && ! $data['valid'] ) {
			return new \WP_Error( 'invalid', 'Invalid license key.' );
		}

		return $data;
	}

	public function activate( array $args = [] ): array|\WP_Error {
		if ( empty( $args['license_key'] ) ) {
			return new \WP_Error( 400, 'Invalid license key.' );
		}
		return Supablox_Api::get_instance()->activate_license( $args );
	}

	public function deactivate( array $args = [] ): array|\WP_Error {
		$license_key = $this->get_license_key();
		$instance_id = $this->get_instance_id();

		if ( empty( $license_key ) || empty( $instance_id ) ) {
			return new \WP_Error( 400, 'Invalid license key or instance ID.' );
		}

		return Supablox_Api::get_instance()->deactivate_license(
			[
				'license_key' => $license_key,
				'instance_id' => $instance_id,
			]
		);
	}

	public function get_latest_version( array $args = [] ): array|\WP_Error {
		$license_key = $this->get_license_key();
		$instance_id = $this->get_instance_id();

		if ( empty( $license_key ) || empty( $instance_id ) ) {
			return new \WP_Error( 400, 'Invalid license key or instance ID.' );
		}

		return Supablox_Api::get_instance()->get_latest_version(
			[
				'license_key' => $license_key,
				'instance_id' => $instance_id,
			]
		);
	}

	public function get_version( string $version, array $args = [] ): array|\WP_Error {
		$license_key = $this->get_license_key();
		$instance_id = $this->get_instance_id();

		if ( empty( $license_key ) || empty( $instance_id ) ) {
			return new \WP_Error( 400, 'Invalid license key or instance ID.' );
		}

		return Supablox_Api::get_instance()->get_version(
			$version,
			[
				'license_key' => $license_key,
				'instance_id' => $instance_id,
			]
		);
	}

	public function is_activated(): bool {
		return ! empty( $this->get_instance_id() ) && ! empty( $this->get_license_key() );
	}

	private function get_instance_id(): string {
		return $this->get_activation()['instance_id'] ?? '';
	}

	private function get_license_key(): string {
		return get_options()['license_key'] ?? '';
	}
}
