<?php

namespace Supablox\Packages\Admin;

class Settings_Page {

	const ID = 'supablox-settings';
	const OPTION_GROUP = 'supablox_settings_option_group';
	const SCREEN_ID = 'settings_page_supablox';

	private array $release_channels = [
		'stable' => [
			'label' => 'Stable',
			'description' => 'Recommended for most users.',
		],
		'beta' => [
			'label' => 'Beta',
			'description' => 'Get early access to new features, but expect bugs. Not recommended for production sites.',
		],
	];

	public function init() {
		add_action( 'admin_menu', [ $this, 'admin_menu' ] );
		add_action( 'admin_init', [ $this, 'register_settings' ] );
		add_action( 'load-' . self::SCREEN_ID, [ $this, 'load' ] );
	}

	public function admin_menu() {
		global $submenu;

		$slugs = isset( $submenu['options-general.php'] )
			? wp_list_pluck( $submenu['options-general.php'], 2 )
			: [];

		if ( ! in_array( 'supablox', $slugs, true ) ) {
			add_submenu_page(
				'options-general.php',
				'Supablox Settings',
				'Supablox',
				'manage_options',
				'supablox',
				[ $this, 'settings_page' ]
			);
			add_filter( 'admin_footer_text', [ $this, 'footer_text' ] );
		}
	}

	public function load() {
		$screen = get_current_screen();

		$content = '';

		foreach ( $this->release_channels as $value => $args ) {
			$content .= sprintf(
				'<p><strong>%s</strong>: %s</p>',
				$args['label'],
				$args['description']
			);
		}

		if ( $this->is_development() ) {
			$content .= sprintf(
				'<p><strong>%s</strong>: %s</p>',
				'Pre-release updates',
				'Contains latest changes from the current release channel.'
			);
		}

		$screen->add_help_tab(
			[
				'id' => self::ID . '-release-channel',
				'title' => __( 'Release channel', 'supablox' ),
				'content' => $content,
			]
		);
	}

	public function settings_page() {
		echo '<div class="wrap">';
		echo '<h1>' . esc_html__( 'Supablox Settings', 'supablox' ) . '</h1>';
		echo '<form class="supablox-settings" method="post" action="options.php">';
		settings_fields( self::OPTION_GROUP );
		do_settings_sections( self::ID );
		submit_button();
		echo '<style>';
		echo '.supablox-settings > h2:not(:first-of-type) { padding-top: 2em; border-top: 1px solid rgba(0,0,0,0.07); }';
		echo '</style>';
		echo '</form>';
		echo '</div>';
	}

	public function footer_text( $html ) {
		if (
			get_current_screen()->id === self::SCREEN_ID &&
			! isset( $_GET['development'] ) && // phpcs:ignore WordPress.Security
			current_user_can( 'manage_options' )
		) {
			$url = add_query_arg( 'development', 1, admin_url( 'options-general.php?page=supablox' ) );
			return wp_sprintf( '%s <a href="%s">Show development settings.</a>', $html, esc_url( $url ) );
		}
		return $html;
	}

	public function register_settings() {
		$plugin_data = get_plugin_data( PACKAGE_FILE );

		register_setting( self::OPTION_GROUP, PACKAGE_NAME, [ $this, 'sanitize_callback' ] );
		add_settings_section( PACKAGE_NAME, $plugin_data['Name'], null, self::ID );

		License::get_instance()->add_settings();
		Features::get_instance()->add_settings();

		add_settings_field( 'release_channel', 'Release channel', [ $this, 'release_channel_callback' ], self::ID, PACKAGE_NAME );
	}

	public function sanitize_callback( $input ) {
		return array_merge(
			License::get_instance()->settings_sanitize_callback( $input ),
			Features::get_instance()->settings_sanitize_callback( $input ),
			[
				'release_channel' => sanitize_key( $input['release_channel'] ?? 'stable' ),
				'prerelease' => isset( $input['prerelease'] ) ? true : false,
			]
		);
	}

	public function release_channel_callback() {
		$options = get_option( PACKAGE_NAME );
		$channel = isset( $options['release_channel'] ) ? $options['release_channel'] : '';

		if ( ! in_array( $channel, array_keys( $this->release_channels ), true ) ) {
			$channel = 'stable';
		}

		printf( '<div style="display:flex;align-items:baseline;gap:16px;">' );

		printf( '<select id="%1$s[release_channel]" name="%1$s[release_channel]" value="%2$s" style="margin-bottom:16px;">', esc_attr( PACKAGE_NAME ), esc_attr( $channel ) );
		foreach ( $this->release_channels as $value => $args ) {
			printf( '<option value="%s" %s>%s</option>', esc_attr( $value ), selected( $value, $channel, false ), esc_attr( $args['label'] ) );
		}
		printf( '</select>' );

		if ( ( isset( $options['prerelease'] ) && $options['prerelease'] ) || $this->is_development() ) {
			printf( '<label><input type="checkbox" id="%1$s[prerelease]" name="%1$s[prerelease]" value="1" %2$s> Include pre-release updates</label>', esc_attr( PACKAGE_NAME ), checked( $options['prerelease'] ?? false, true, false ) );
		}

		printf( '</div>' );
	}

	private function is_development() {
		return isset( $_GET['development'] ) && $_GET['development'] === '1'; // phpcs:ignore WordPress.Security.NonceVerification
	}
}
