<?php

namespace Supablox;

$_supablox_preview_templates_map = [];
$_supablox_preview_post_term_map = [];

if ( is_preview() ) {
	global $post;

	require_once ABSPATH . WPINC . '/pluggable.php';

	if ( $post && ! current_user_can( 'edit_post', $post ) ) {
		wp_die( 'Unauthorized.', 'Unauthorized', [ 'response' => 403 ] );
	} elseif ( ! current_user_can( 'edit_posts' ) ) {
		wp_die( 'Unauthorized.', 'Unauthorized', [ 'response' => 403 ] );
	}

	// phpcs:ignore WordPress.Security.NonceVerification
	if ( isset( $_GET['supablox_preview_close'] ) ) {
		delete_transient( get_preview_key() );
		die;
	}

	add_action( 'show_admin_bar', '__return_false' );
	add_action( 'init', __NAMESPACE__ . '\preview_init', 9 );
	add_filter( 'wp_theme_json_data_user', __NAMESPACE__ . '\preview_override_theme_json' );
	add_filter( 'the_posts', __NAMESPACE__ . '\preview_override_posts', 10, 2 );
	add_filter( 'get_the_terms', __NAMESPACE__ . '\preview_override_terms', 10, 3 );
}

function preview_init() {
	$data = get_preview_data();

	if ( ! $data ) return;

	foreach ( [ 'wp_template', 'wp_template_part' ] as $post_type ) {
		if ( isset( $data['postTypes'][ $post_type ] ) ) {
			create_preview_posts( $data['postTypes'][ $post_type ], $post_type );
		}
	}

	if ( isset( $data['options'] ) ) {
		foreach ( $data['options'] as $key => $value ) {
			add_filter( "option_{$key}", fn() => $value );
		}
	}

	do_action( 'supablox_preview_init', $data );
}

function create_preview_posts( array $records, string $post_type ) {
	global $_supablox_preview_templates_map;
	global $_supablox_preview_post_term_map;

	foreach ( $records as $key => $value ) {
		list ( $theme, $slug ) = explode( '//', $key );

		$post_id = $value['id'] ?: get_next_post_id();

		$post = new \WP_Post(
			(object) [
				'ID' => $post_id,
				'post_type' => $post_type,
				'post_name' => $slug,
				'post_parent' => 0,
				'post_status' => 'publish',
				'post_title' => $value['title'],
				'post_content' => $value['content'],
			]
		);

		$term = new \WP_Term(
			(object) [
				'term_id' => get_next_term_id(),
				'name' => $theme,
			]
		);

		$_supablox_preview_templates_map[ $key ] = $post_id;
		$_supablox_preview_post_term_map[ $post_id ] = $term->term_id;

		wp_cache_set( $post->ID, $post, 'posts' );
		wp_cache_set( $term->term_id, $term, 'terms' );
	}
}

function preview_override_theme_json( \WP_Theme_JSON_Data $theme_json ) {
	$data = get_preview_data();

	if ( empty( $data['globalStyles'] ) ) {
		return $theme_json;
	}

	return new \WP_Theme_JSON_Data(
		[
			'version' => \WP_Theme_JSON::LATEST_SCHEMA,
			...$data['globalStyles'],
		],
		'user'
	);
}

function preview_override_posts( array $posts, \WP_Query $query ) {
	if (
		empty( $posts ) &&
		isset( $query->query_vars['term_id'] ) &&
		isset( $query->query_vars['post_type'] ) &&
		isset( $query->query_vars['post_name__in'] ) &&
		in_array( $query->query_vars['post_type'], [ 'wp_template', 'wp_template_part' ], true )
	) {
		return array_filter(
			array_map(
				function ( $slug ) use ( $query ) {
					global $_supablox_preview_templates_map;
					$theme = $query->query_vars['term_id'];
					$key = "{$theme}//{$slug}";
					$post_id = $_supablox_preview_templates_map[ $key ] ?? 0;
					return wp_cache_get( $post_id, 'posts' );
				},
				$query->query_vars['post_name__in']
			),
			'is_object'
		);
	}

	$data = get_preview_data();

	foreach ( $posts as $post ) {
		$post_data = $data['postTypes'][ $post->post_type ][ $post->ID ] ?? null;

		if ( $post_data ) {
			$post->post_title = $post_data['title'];
			$post->post_content = $post_data['content'];
			$post->post_excerpt = $post_data['excerpt'];
		}
	}

	return $posts;
}

function preview_override_terms( $terms, $post_id, $taxonomy ) {
	global $_supablox_preview_post_term_map;

	if ( isset( $_supablox_preview_post_term_map[ $post_id ] ) ) {
		return [ wp_cache_get( $_supablox_preview_post_term_map[ $post_id ], 'terms' ) ];
	}

	return $terms;
}

function get_preview_key() {
	$current_user = wp_get_current_user();
	return "supablox_preview_{$current_user->ID}";
}

function get_preview_data() {
	return get_transient( get_preview_key() );
}

function get_next_post_id() {
	static $next_id = null;
	if ( $next_id === null ) {
		global $wpdb;
		$next_id = (int) $wpdb->get_var( "SELECT MAX(ID) FROM $wpdb->posts" ) + 1;
	}
	return $next_id++;
}

function get_next_term_id() {
	static $next_id = null;
	if ( $next_id === null ) {
		global $wpdb;
		$next_id = (int) $wpdb->get_var( "SELECT MAX(term_id) FROM $wpdb->terms" ) + 1;
	}
	return $next_id++;
}
