<?php

namespace Supablox;

add_action( 'rest_api_init', __NAMESPACE__ . '\rest_api_init' );
add_filter( 'rest_endpoints', __NAMESPACE__ . '\rest_endpoints' );
add_filter( 'wp_rest_search_handlers', __NAMESPACE__ . '\rest_register_search_handler' );

function rest_api_init() {
	require_once ABSPATH . 'wp-admin/includes/post.php';

	$preview_controller = new REST_Preview_Controller();
	$settings_controller = new REST_Settings_Controller();

	$preview_controller->register_routes();
	$settings_controller->register_routes();

	add_filter( 'rest_post_search_query', __NAMESPACE__ . '\rest_post_search_query', 10, 2 );
	add_filter( 'get_block_templates', __NAMESPACE__ . '\rest_process_block_templates' );

	register_rest_field(
		'page',
		'_is_page_on_front',
		[
			'get_callback' => fn( $post ) => is_page_on_front( $post ),
		]
	);

	register_rest_field(
		'page',
		'_is_page_for_posts',
		[
			'get_callback' => fn( $post ) => is_page_for_posts( $post ),
		]
	);

	register_rest_field(
		'search-result',
		'_is_page_on_front',
		[
			'get_callback' => fn( $post ) => is_page_on_front( $post ),
		]
	);

	register_rest_field(
		'search-result',
		'_slug',
		[
			'get_callback' => fn( $post ) => get_post_field( 'post_name', $post['id'] ),
		]
	);

	$post_type_names = array_map(
		fn ( \WP_Post_Type $post_type ) => $post_type->name,
		get_supported_post_types()
	);

	foreach ( [ 'search-result', ...$post_type_names ] as $object_type ) {
		register_rest_field(
			$object_type,
			'_supablox_edit_link',
			[
				'get_callback' => function ( $post ) {
					if ( in_array( $post['type'], [ 'wp_template', 'wp_template_part' ], true ) ) {
						return get_editor_url(
							[
								'postType' => $post['type'],
								'postId' => $post['id'],
							]
						);
					}
					return get_edit_post_link( $post['id'] );
				},
			]
		);

		register_rest_field(
			$object_type,
			'_supablox_post_lock',
			[
				'get_callback' => function ( $post ) {
					$user_id = wp_check_post_lock( $post['id'] );
					$user = get_userdata( $user_id );
					if ( $user_id && $user ) {
						return [
							'avatar' => get_avatar_url( $user_id, [ 'size' => 128 ] ),
							'name' => $user->display_name,
						];
					}
				},
			]
		);
	}
}

function rest_endpoints( $endpoints ) {
	$routes = [
		'/wp/v2/global-styles/themes/(?P<stylesheet>[^\\/:<>\\*\\?\"\\|]+(?:\\/[^\\/:<>\\*\\?\"\\|]+)?)',
		'/wp/v2/global-styles/(?P<id>[\\/\\w-]+)',
		'/wp/v2/users/(?P<id>[\\d]+)',
		'/wp/v2/navigation',
		'/wp/v2/settings',
		'/wp-block-editor/v1/navigation-fallback',
	];

	foreach ( $routes as $route ) {
		if ( ! isset( $endpoints[ $route ] ) ) {
			continue;
		}

		foreach ( $endpoints[ $route ] as $key => &$value ) {
			if ( ! is_int( $key ) ) continue;
			if ( ! is_array( $value ) ) continue;

			if ( $value['methods'] === 'GET' ) {
				rest_override_permission_callback( $value );

				if ( $route === '/wp/v2/navigation' ) {
					// The editor sends an array of statuses when not logged in as admin.
					$value['args']['status'] = [ 'type' => null ];
				}
			}
		}
	}

	return $endpoints;
}

function rest_override_permission_callback( array &$handler ) {
	$permission_callback = $handler['permission_callback'] ?? null;

	if ( is_callable( $permission_callback ) ) {
		$handler['permission_callback'] = function () use ( $permission_callback ) {
			if ( current_user_can( 'edit_posts' ) ) {
				return true;
			}
			foreach ( get_post_types( [ 'show_in_rest' => true ], 'objects' ) as $post_type ) {
				if ( current_user_can( $post_type->cap->edit_posts ) ) {
					return true;
				}
			}
			if ( current_user_can( 'edit_theme_options' ) ) {
				return true;
			}
			return call_user_func( $permission_callback );
		};
	}
}

function rest_register_search_handler( array $handlers ) {
	if ( isset( $_GET['supablox_search'] ) ) { // phpcs:ignore WordPress.Security
		$handlers[] = new REST_Search_Handler();
	}
	return $handlers;
}

/**
 * @param array<\WP_Block_Template> $query_result
 */
function rest_process_block_templates( array $query_result ) {
	foreach ( $query_result as $template ) {
		if ( $template->wp_id && empty( $template->content ) ) {
			// Add non-empty content to empty templates so that the editor
			// is able to resolve them when looking up the current template.
			$template->content = ' ';
		}
	}
	return $query_result;
}

/**
 * Include all post statuses in the search query if the request is from the Supablox editor.
 */
function rest_post_search_query( array $query_args, \WP_REST_Request $req ) {
	if ( $req->get_param( 'supablox_search' ) && wp_verify_nonce( $req->get_header( 'X-Wp-Nonce' ), 'wp_rest' ) ) {
		$query_args['post_status'] = [ 'publish', 'future', 'draft', 'pending', 'private' ];
	}
	return $query_args;
}
